#! /usr/bin/env python
from __future__ import print_function

from tigres import *


#######################
# Functions used for
# Task implementations
#######################

def f_task1(text):
    return text + "1, "


def f_task2(text=None):
    """
    This task can handle streaming input
    """
    if text:
        print("t2: %s, %s" % (text, type(text)))
    else:
        # No inputs, look for data in stdin
        import fileinput
        text = ""
        for line in fileinput.input():
            text += line
    return text + "2, "


def f_task3(text=None):
    """
    This task can handle streaming input
    """
    if text:
        print("t3: %s, %s" % (text, type(text)))
    else:
        # No inputs, look for data in stdin
        import fileinput
        text = ""
        for line in fileinput.input():
            text += line
    return text + "3"


######################
# Main Workflow
######################
def main():
    # This example shows how to use:
    #   Sequence
    #   Sequence with redirection
    #   Parallel
    #   Parallel Sequential
    #   Parallel Sequentia with redirection
    start(name="example_parallel_sequential", log_dest='example_parallel_sequential.log')

    ###################
    # Create the Tasks
    ###################

    task1 = Task("task1", FUNCTION, f_task1, [str])
    task2 = Task("task2", FUNCTION, f_task2, [])
    task3 = Task("task3", FUNCTION, f_task3, [])
    tasks = TaskArray("Lane", [task1, task2, task3])

    #######################
    #     Sequence
    #######################
    inputs = InputArray("S0", [["s-one"]])

    print("\nRunning sequence")
    output_seq = sequence("sequential", tasks, inputs)
    print_outputs(output_seq, "s out: %s")

    tasks = TaskArray("Lane", [task1, task2, task3])

    ############################
    # Sequence with redirection
    ############################
    print("\nRunning sequence redirect")
    outputs = sequence("sequential_pipeline", tasks, inputs, redirect=True)
    assert (output_seq == outputs)
    print_outputs(outputs, "s pipeline out: %s")

    #######################
    #     Parallel
    #######################
    inputs = InputArray("P0", [["p-one"], ["p-two"], ["p-three"]])

    print("\nRunning parallel")
    outputs = parallel("parallel", tasks, inputs)
    print_outputs(outputs, "p out: %s")

    #######################
    # Parallel Sequential
    #######################
    inputs = InputArray("PS0", [["one"], ["two"], ["three"], ["four"], ["five"], ["six"], ["seven"], ["eight"]])

    print("\nRunning parallel_sequential")
    ps_outputs = parallel_sequential("parallel_sequential", tasks, inputs)
    print_outputs(ps_outputs, "ps out: %s")

    #######################################
    # Parallel Sequential with redirection
    #######################################
    print("\nRunning parallel_sequential redirect")
    outputs = parallel_sequential("parallel_sequential_pipeline", tasks, inputs, redirect=True)

    print_outputs(outputs, "ps pipeline out: %s")
    assert (ps_outputs == outputs)

    dot_execution()


def print_outputs(outputs, prefix):
    """
    Print the outputs with the given prefix
    """
    if isinstance(outputs, list):
        for output in outputs:
            print(prefix % str(output))
    else:
        print(prefix % outputs)


if __name__ == "__main__":
    main()
