#! /usr/bin/env python
from __future__ import print_function

import os
import sys

from tigres import *
from tigres.utils import Execution


# Workflow

def main(execution):
    # This example tests the parallel_sequential template

    start(name="example_parallel_sequential_external", log_dest='example_parallel_sequential.log',
          execution=execution)

    outputs = "output"

    task1 = Task("task1", EXECUTABLE, "{}/../task1_ps.py".format(os.getcwd()), [str])
    task2 = Task("task2", EXECUTABLE, "{}/../task2_ps.py".format(os.getcwd()), [str])
    task3 = Task("task3", EXECUTABLE, "{}/../task3_ps.py".format(os.getcwd()), [str])

    tasks = TaskArray("Lane", [task1, task2, task3])
    # tasks = TaskArray("Lane", [task1, task2])

    # first try sequential
    # v_p   = InputValues("PREVIOUS", [PREVIOUS])
    # input = InputArray("S0", [["s-one"], ["s-two"]])
    input = InputArray("S0", [["s-one"], [PREVIOUS], [PREVIOUS]])

    print("Running sequence")
    outputs = sequence("sequential", tasks, input)

    if isinstance(outputs, list):
        for output in outputs:
            print("s out: %s" % str(output))
    else:
        print("s out: " + outputs)

    # next try parallel
    input = InputArray("P0", [["p-one"], ["p-two"], ["p-three"]])

    # print "Running parallel"
    # outputs = parallel("parallel", tasks, input)

    if isinstance(outputs, list):
        for output in outputs:
            print("p out: %s" % str(output))
    else:
        print("p out: " + outputs)

    # sys.exit(0)

    # next try parallel_sequential
    input = InputArray("PS0", [["one"], ["two"], ["three"], ["four"], ["five"], ["six"], ["seven"], ["eight"]])
    # input = InputArray("PS0", [["one"], ["two"]])

    print("Running parallel_sequential")
    outputs = parallel_sequential("parallel_sequential", tasks, input)

    if isinstance(outputs, list):
        for output in outputs:
            print("ps out: %s" % str(output))
    else:
        print("ps out: " + str(outputs))

    dot_execution()


if __name__ == "__main__":

    if len(sys.argv) <= 1:
        print(("Usage: {} ({})>".format(sys.argv[0], "|".join(list(Execution.LOOKUP.keys())))))
        exit()

    main(Execution.get(sys.argv[1]))
