#! /usr/bin/env python
from __future__ import print_function

import sys

from tigres import *
from tigres.utils import Execution


# Functions

def f_task1(text):
    return text + "1, "


def f_task2(text=None):
    """
    This task can handle streaming input
    """
    if text:
        print("t2: %s, %s" % (text, type(text)))
    else:
        import fileinput
        text = ""
        for line in fileinput.input():
            text += line
    return text + "2, "


def f_task3(text=None):
    """
    This task can handle streaming input
    """
    if text:
        print("t3: %s, %s" % (text, type(text)))
    else:
        import fileinput
        text = ""
        for line in fileinput.input():
            text += line
    return text + "3"


# Workflow

def main(execution):
    # This example tests the parallel_sequential template

    start(name="example_parallel_sequential", log_dest='example_parallel_sequential.log',
          execution=execution)

    outputs = "output"

    task1 = Task("task1", FUNCTION, f_task1, [str])

    task2 = Task("task2", FUNCTION, f_task2, [])
    task3 = Task("task3", FUNCTION, f_task3, [])
    tasks = TaskArray("Lane", [task1, task2, task3])

    # first try sequential
    input = InputArray("S0", [["s-one"]])

    print("\nRunning sequence")
    output_seq = sequence("sequential", tasks, input)
    prefix = "s out: %s"

    print_outputs(output_seq, prefix)

    tasks = TaskArray("Lane", [task1, task2, task3])

    print("\nRunning sequence pipeline")
    outputs = sequence("sequential_pipeline", tasks, input, redirect=True)
    prefix = "s pipeline out: %s"

    assert (output_seq == outputs)

    print_outputs(outputs, prefix)

    # next try parallel
    input = InputArray("P0", [["p-one"], ["p-two"], ["p-three"]])

    print("\nRunning parallel")
    outputs = parallel("parallel", tasks, input)

    print_outputs(outputs, "p out: %s")

    # return

    # next try parallel_sequential
    input = InputArray("PS0", [["one"], ["two"], ["three"], ["four"], ["five"], ["six"], ["seven"], ["eight"]])
    # input = InputArray("PS0", [["one"], ["two"]])

    print("\nRunning parallel_sequential")
    ps_outputs = parallel_sequential("parallel_sequential", tasks, input)

    print_outputs(ps_outputs, "ps out: %s")

    print("\nRunning parallel_sequential pipeline")
    outputs = parallel_sequential("parallel_sequential_pipeline", tasks, input, redirect=True)

    print_outputs(outputs, "ps pipeline out: %s")

    assert (ps_outputs == outputs)

    dot_execution()


def print_outputs(outputs, prefix):
    if isinstance(outputs, list):
        for output in outputs:
            print(prefix % str(output))
    else:
        print(prefix % outputs)


if __name__ == "__main__":

    if len(sys.argv) <= 1:
        print(("Usage: {} ({})>".format(sys.argv[0], "|".join(list(Execution.LOOKUP.keys())))))
        exit()

    main(Execution.get(sys.argv[1]))
