__author__ = 'foxjas09'

from tigres import *
from tigres.utils import Execution, State, TigresException
import os

EXEC_MODE = 'EXECUTION_LOCAL_THREAD'
THIS_DIR = os.path.dirname(__file__)
NORMAL_TASK = os.path.join(THIS_DIR, 'normal_task.sh')
NORMAL_DIVIDE = os.path.join(THIS_DIR, 'normal_divide.sh')
DIVIDE_BY_ZERO = os.path.join(THIS_DIR, 'divide_by_zero.sh')

LOG_DEST = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'broken_sequence.log')

def run_broken_sequence(recover=False, recovery_log=True):
    start(name="BrokenSequence", log_dest=LOG_DEST, execution=Execution.get(EXEC_MODE), recover=recover, recovery_log=recovery_log)

    all_tasks = TaskArray("All tasks")
    all_tasks.append(Task("normal task 1", EXECUTABLE, impl_name=NORMAL_TASK))
    all_tasks.append(Task("normal task 2", EXECUTABLE, impl_name=NORMAL_TASK))
    all_tasks.append(Task("broken task", EXECUTABLE, impl_name=DIVIDE_BY_ZERO))
    all_tasks.append(Task("normal task 3", EXECUTABLE, impl_name=NORMAL_TASK))
    all_tasks_inputs = InputArray("Broken Sequence inputs", [])
    for i in range(1, 4):
        all_tasks_inputs.append(InputValues("number", [i]))
    all_tasks_inputs[2] = InputValues()  # 3rd task takes no input
    try:
        output = sequence('run broken sequence', all_tasks,
                          all_tasks_inputs)
    except TigresException as e:
        print(e)

    dot_execution()
    print("FAILED TASKS:\n")
    fail_records, done_records = check('task', state=State.FAIL), check('task', state=State.DONE)
    if fail_records:
        for status in check('task', state=State.FAIL):
            print("{}\n".format(status))
    print("DONE TASKS:\n")
    if done_records:
        for status in check('task', state=State.DONE):
            print("{}\n".format(status))

def run_fixed_sequence(recover=True, recovery_log=True):
    start(name="FixedSequence", log_dest=LOG_DEST, execution=Execution.get(EXEC_MODE), recover=recover, recovery_log=recovery_log)

    all_tasks = TaskArray("All tasks")
    all_tasks.append(Task("normal task 1", EXECUTABLE, impl_name=NORMAL_TASK))
    all_tasks.append(Task("normal task 2", EXECUTABLE, impl_name=NORMAL_TASK))
    all_tasks.append(Task("broken task", EXECUTABLE, impl_name=NORMAL_DIVIDE))
    all_tasks.append(Task("normal task 3", EXECUTABLE, impl_name=NORMAL_TASK))
    all_tasks_inputs = InputArray("Broken Sequence inputs", [])
    for i in range(4):
        all_tasks_inputs.append(InputValues("number", [i]))
    all_tasks_inputs[2] = InputValues()  # 3rd task takes no input
    try:
        output = sequence('run broken sequence', all_tasks, all_tasks_inputs)
    except TigresException as e:
        print(e)

    dot_execution()
    print("FAILED TASKS:\n")
    fail_records, done_records = check('task', state=State.FAIL), check('task', state=State.DONE)
    if fail_records:
        for status in check('task', state=State.FAIL):
            print("{}\n".format(status))
    print("DONE TASKS:\n")
    if done_records:
        for status in check('task', state=State.DONE):
            print("{}\n".format(status))

def run_fixed_sequence_m(recover=True, recovery_log=True):
    """
    Fixed broken task, except task(s) prior to it have been changed
    """
    start(name="FixedSequence", log_dest=LOG_DEST, execution=Execution.get(EXEC_MODE), recover=recover, recovery_log=recovery_log)

    all_tasks = TaskArray("All tasks")
    all_tasks.append(Task("normal task 1", EXECUTABLE, impl_name=NORMAL_TASK))
    all_tasks.append(Task("different task 2", EXECUTABLE, impl_name=NORMAL_TASK))
    all_tasks.append(Task("broken task", EXECUTABLE, impl_name=NORMAL_DIVIDE))
    all_tasks.append(Task("different task 3", EXECUTABLE, impl_name=NORMAL_TASK))
    all_tasks_inputs = InputArray("Broken Sequence inputs", [])
    for i in range(4):
        all_tasks_inputs.append(InputValues("number", [i]))
    all_tasks_inputs[2] = InputValues()  # 3rd task takes no input
    try:
        output = sequence('run broken sequence', all_tasks, all_tasks_inputs)
    except TigresException as e:
        print(e)

    dot_execution()
    print("FAILED TASKS:\n")
    fail_records, done_records = check('task', state=State.FAIL), check('task', state=State.DONE)
    if fail_records:
        for status in check('task', state=State.FAIL):
            print("{}\n".format(status))
    print("DONE TASKS:\n")
    if done_records:
        for status in check('task', state=State.DONE):
            print("{}\n".format(status))

if __name__ == "__main__":
    # Simple Usage Here
    run_broken_sequence()
    run_fixed_sequence()
    #run_fixed_sequence_m()