__author__ = 'foxjas09'

from tigres import *
from tigres.utils import Execution, State, TigresException
import os

EXEC_MODE = 'EXECUTION_LOCAL_THREAD'
THIS_DIR = os.path.dirname(__file__)
NORMAL_TASK = os.path.join(THIS_DIR, 'normal_task.sh')
NORMAL_DIVIDE = os.path.join(THIS_DIR, 'normal_divide.sh')
DIVIDE_BY_ZERO = os.path.join(THIS_DIR, 'divide_by_zero.sh')

LOG_DEST_SM = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'broken_split_merge.log')

def run_broken_split_parallel(recover=False, recovery_log=True):
    start(name="BrokenSplitParallel", log_dest=LOG_DEST_SM, execution=Execution.get(EXEC_MODE), recover=recover, recovery_log=recovery_log)
    broken_split = Task("Split", FUNCTION, impl_name=lambda x: str(x), input_types=[int])
    broken_split_input = InputValues("number", [123])
    after_split_tasks = TaskArray("After split")
    after_split_tasks.append(Task("normal task 1", EXECUTABLE, impl_name=NORMAL_TASK, input_types=[str]))
    after_split_tasks.append(Task("variable task 2", EXECUTABLE, impl_name=DIVIDE_BY_ZERO, input_types=[str])) # fails
    after_split_tasks.append(Task("normal task 3", EXECUTABLE, impl_name=NORMAL_TASK, input_types=[str]))

    try:
        output = split('Run BrokenAfterSplit', broken_split, broken_split_input, after_split_tasks,
                       InputArray("Iterable", []))
        print(output)
    except TigresException as e:
        print(e)

    dot_execution()
    check_final_statuses()

    end()

def run_fixed_split_parallel(recover=True, recovery_log=True):
    start(name="FixedSplitParallel", log_dest=LOG_DEST_SM, execution=Execution.get(EXEC_MODE), recover=recover, recovery_log=recovery_log)
    """ Split phase """
    broken_split = Task("Split", FUNCTION, impl_name=lambda x: str(x), input_types=[int])
    broken_split_input = InputValues("number", [123])
    after_split_tasks = TaskArray("After split")
    after_split_tasks.append(Task("normal task 1", EXECUTABLE, impl_name=NORMAL_TASK, input_types=[str]))
    after_split_tasks.append(Task("variable task 2", EXECUTABLE, impl_name=NORMAL_TASK, input_types=[str]))
    after_split_tasks.append(Task("normal task 3", EXECUTABLE, impl_name=NORMAL_TASK, input_types=[str]))

    try:
        output = split('Run BrokenAfterSplit', broken_split, broken_split_input, after_split_tasks,
                       InputArray("Iterable", []))
        print(output)
    except TigresException as e:
        print(e)

    """ Merge phase """
    identity_task = Task("Identity function", FUNCTION, impl_name=lambda x: x, input_types=[str])
    merge_task_array = TaskArray('Output all inputs', tasks=[identity_task])
    output = merge('Merge', merge_task_array, None, \
                   Task("Join strings", FUNCTION, impl_name=lambda x: '; '.join(x), input_types=[str]))

    dot_execution()
    check_final_statuses()
    end()

def run_broken_merge_parallel(recover=False, recovery_log=True):
    start(name="BrokenMergeParallel", log_dest=LOG_DEST_SM, execution=Execution.get(EXEC_MODE), recover=recover, recovery_log=recovery_log)

    """ Split phase """
    broken_split = Task("Split", FUNCTION, impl_name=lambda x: str(x), input_types=[int])
    broken_split_input = InputValues("number", [123])
    after_split_tasks = TaskArray("After split")
    after_split_tasks.append(Task("Task 1", EXECUTABLE, impl_name=NORMAL_TASK, input_types=[str]))
    after_split_tasks.append(Task("Task 2", EXECUTABLE, impl_name=NORMAL_TASK, input_types=[str]))
    after_split_tasks.append(Task("Task 3", EXECUTABLE, impl_name=NORMAL_TASK, input_types=[str]))

    output = split('Run SplitParallel', broken_split, broken_split_input, after_split_tasks, InputArray("Iterable", []))
    print(output)

    """ Merge phase """
    identity_task1 = Task("Task 4", FUNCTION, impl_name=lambda x: x, input_types=[str])
    identity_task2 = Task("Task 5", FUNCTION, impl_name=lambda x: x, input_types=[str])
    broken_task = Task("Task 6", FUNCTION, impl_name=lambda x: x / 0, input_types=[str])
    merge_task_array = TaskArray('Output all inputs', tasks=[identity_task1, identity_task2, broken_task])

    try:
        output = merge('Merge', merge_task_array, None, \
                   Task("Merge Task", FUNCTION, impl_name=lambda x: '; '.join(x), input_types=[str]))
        print(output)
    except TigresException as e:
        print(e)

    dot_execution()
    check_final_statuses()
    end()

def run_fixed_merge_parallel(recover=True, recovery_log=True):
    start(name="FixedMergeParallel", log_dest=LOG_DEST_SM, execution=Execution.get(EXEC_MODE), recover=recover, recovery_log=recovery_log)

    """ Split phase """
    broken_split = Task("Split", FUNCTION, impl_name=lambda x: str(x), input_types=[int])
    broken_split_input = InputValues("number", [123])
    after_split_tasks = TaskArray("After split")
    after_split_tasks.append(Task("Task 1", EXECUTABLE, impl_name=NORMAL_TASK, input_types=[str]))
    after_split_tasks.append(Task("Task 2", EXECUTABLE, impl_name=NORMAL_TASK, input_types=[str]))
    after_split_tasks.append(Task("Task 3", EXECUTABLE, impl_name=NORMAL_TASK, input_types=[str]))

    output = split('Run SplitParallel', broken_split, broken_split_input, after_split_tasks, InputArray("Iterable", []))
    print(output)

    """ Merge phase """
    identity_task1 = Task("Task 4", FUNCTION, impl_name=lambda x: x, input_types=[str])
    identity_task2 = Task("Task 5", FUNCTION, impl_name=lambda x: x, input_types=[str])
    identity_task3 = Task("Task 6", FUNCTION, impl_name=lambda x: x, input_types=[str]) # fixed
    merge_task_array = TaskArray('Output all inputs', tasks=[identity_task1, identity_task2, identity_task3])

    try:
        output = merge('Merge', merge_task_array, None, \
                   Task("Merge Task", FUNCTION, impl_name=lambda x: '; '.join(x), input_types=[str]))
        print(output)
    except TigresException as e:
        print(e)

    dot_execution()
    check_final_statuses()
    end()

def run_fixed_merge_parallel_m(recover=True, recovery_log=True):
    """
    Tests situation where tasks prior to the fixed task have been changed.
    """
    start(name="FixedMergeParallel", log_dest=LOG_DEST_SM, execution=Execution.get(EXEC_MODE), recover=recover, recovery_log=recovery_log)

    """ Split phase """
    broken_split = Task("Different Split", FUNCTION, impl_name=lambda x: str(x), input_types=[int])
    broken_split_input = InputValues("number", [123])
    after_split_tasks = TaskArray("After split")
    after_split_tasks.append(Task("Different Task 1", EXECUTABLE, impl_name=NORMAL_TASK, input_types=[str]))
    after_split_tasks.append(Task("Different Task 2", EXECUTABLE, impl_name=NORMAL_TASK, input_types=[str]))
    after_split_tasks.append(Task("Task 3", EXECUTABLE, impl_name=NORMAL_TASK, input_types=[str]))

    output = split('Run SplitParallel', broken_split, broken_split_input, after_split_tasks, InputArray("Iterable", []))
    print(output)

    """ Merge phase """
    identity_task1 = Task("Task 4", FUNCTION, impl_name=lambda x: x, input_types=[str])
    identity_task2 = Task("Task 5", FUNCTION, impl_name=lambda x: x, input_types=[str])
    identity_task3 = Task("Task 6", FUNCTION, impl_name=lambda x: x, input_types=[str]) # fixed
    merge_task_array = TaskArray('Output all inputs', tasks=[identity_task1, identity_task2, identity_task3])

    try:
        output = merge('Merge', merge_task_array, None, \
                   Task("Merge Task", FUNCTION, impl_name=lambda x: '; '.join(x), input_types=[str]))
        print(output)
    except TigresException as e:
        print(e)

    dot_execution()
    check_final_statuses()
    end()

def run_broken_merge(recover=False, recovery_log=True):
    start(name="BrokenMerge", log_dest=LOG_DEST_SM, execution=Execution.get(EXEC_MODE), recover=recover, recovery_log=recovery_log)

    """ Split phase """
    broken_split = Task("Split", FUNCTION, impl_name=lambda x: str(x), input_types=[int])
    broken_split_input = InputValues("number", [123])
    after_split_tasks = TaskArray("After split")
    after_split_tasks.append(Task("normal task 1", EXECUTABLE, impl_name=NORMAL_TASK, input_types=[str]))
    after_split_tasks.append(Task("normal task 2", EXECUTABLE, impl_name=NORMAL_TASK, input_types=[str]))
    after_split_tasks.append(Task("normal task 3", EXECUTABLE, impl_name=NORMAL_TASK, input_types=[str]))

    output = split('Run SplitParallel', broken_split, broken_split_input, after_split_tasks, InputArray("Iterable", []))
    print(output)

    """ Merge phase """
    identity_task1 = Task("Identity function_1", FUNCTION, impl_name=lambda x: x, input_types=[str])
    identity_task2 = Task("Identity function_2", FUNCTION, impl_name=lambda x: x, input_types=[str])
    identity_task3 = Task("Identity function_3", FUNCTION, impl_name=lambda x: x, input_types=[str])
    merge_task_array = TaskArray('Output all inputs', tasks=[identity_task1, identity_task2, identity_task3])

    try:
        output = merge('Merge', merge_task_array, None, \
                   Task("Broken join", FUNCTION, impl_name=lambda x: x / 0, input_types=[str]))
        print(output)
    except TigresException as e:
        print(e)

    dot_execution()
    check_final_statuses()
    end()

def run_fixed_merge(recover=True, recovery_log=True):
    start(name="FixedMerge", log_dest=LOG_DEST_SM, execution=Execution.get(EXEC_MODE), recover=recover, recovery_log=recovery_log)

    """ Split phase """
    broken_split = Task("Split", FUNCTION, impl_name=lambda x: str(x), input_types=[int])
    broken_split_input = InputValues("number", [123])
    after_split_tasks = TaskArray("After split")
    after_split_tasks.append(Task("normal task 1", EXECUTABLE, impl_name=NORMAL_TASK, input_types=[str]))
    after_split_tasks.append(Task("normal task 2", EXECUTABLE, impl_name=NORMAL_TASK, input_types=[str]))
    after_split_tasks.append(Task("normal task 3", EXECUTABLE, impl_name=NORMAL_TASK, input_types=[str]))

    output = split('Run SplitParallel', broken_split, broken_split_input, after_split_tasks, InputArray("Iterable", []))
    print(output)

    """ Merge phase """
    identity_task1 = Task("Identity function_1", FUNCTION, impl_name=lambda x: x, input_types=[str])
    identity_task2 = Task("Identity function_2", FUNCTION, impl_name=lambda x: x, input_types=[str])
    identity_task3 = Task("Identity function_3", FUNCTION, impl_name=lambda x: x, input_types=[str])
    merge_task_array = TaskArray('Output all inputs', tasks=[identity_task1, identity_task2, identity_task3])

    try:
        output = merge('Merge', merge_task_array, None, \
                   Task("Broken join", FUNCTION, impl_name=lambda x: '; '.join(x), input_types=[str]))
        print(output)
    except TigresException as e:
        print(e)

    dot_execution()
    check_final_statuses()
    end()


def run_broken_split(recover=False, recovery_log=True):
    start(name="BrokenSplit", log_dest=LOG_DEST_SM, execution=Execution.get(EXEC_MODE), recover=recover, recovery_log=recovery_log)

    """ Split phase """
    broken_split = Task("Broken split", FUNCTION, impl_name=lambda x: x / 0, input_types=[int])
    broken_split_input = InputValues("number", [123])
    after_split_tasks = TaskArray("After split")
    after_split_tasks.append(Task("normal task 1", EXECUTABLE, impl_name=NORMAL_TASK, input_types=[str]))
    after_split_tasks.append(Task("normal task 2", EXECUTABLE, impl_name=NORMAL_TASK, input_types=[str]))
    after_split_tasks.append(Task("normal task 3", EXECUTABLE, impl_name=NORMAL_TASK, input_types=[str]))

    try:
        output = split('Run SplitParallel', broken_split, broken_split_input, after_split_tasks, InputArray("Iterable", []))
        print(output)

        """ Merge phase """
        identity_task1 = Task("Identity function_1", FUNCTION, impl_name=lambda x: x, input_types=[str])
        identity_task2 = Task("Identity function_2", FUNCTION, impl_name=lambda x: x, input_types=[str])
        identity_task3 = Task("Identity function_3", FUNCTION, impl_name=lambda x: x, input_types=[str])
        merge_task_array = TaskArray('Output all inputs', tasks=[identity_task1, identity_task2, identity_task3])

        output = merge('Merge', merge_task_array, None, \
                       Task("String join", FUNCTION, impl_name=lambda x: '; '.join(x), input_types=[str]))
        print(output)
    except TigresException as e:
        print(e)

    dot_execution()
    check_final_statuses()
    end()

def run_fixed_split(recover=True, recovery_log=True):
    start(name="FixedSplit", log_dest=LOG_DEST_SM, execution=Execution.get(EXEC_MODE), recover=recover, recovery_log=recovery_log)

    """ Split phase """
    broken_split = Task("Broken split", FUNCTION, impl_name=lambda x: str(x), input_types=[int])
    broken_split_input = InputValues("number", [123])
    after_split_tasks = TaskArray("After split")
    after_split_tasks.append(Task("normal task 1", EXECUTABLE, impl_name=NORMAL_TASK, input_types=[str]))
    after_split_tasks.append(Task("normal task 2", EXECUTABLE, impl_name=NORMAL_TASK, input_types=[str]))
    after_split_tasks.append(Task("normal task 3", EXECUTABLE, impl_name=NORMAL_TASK, input_types=[str]))

    try:
        output = split('Run SplitParallel', broken_split, broken_split_input, after_split_tasks, InputArray("Iterable", []))
        print(output)

        """ Merge phase """
        identity_task1 = Task("Identity function_1", FUNCTION, impl_name=lambda x: x, input_types=[str])
        identity_task2 = Task("Identity function_2", FUNCTION, impl_name=lambda x: x, input_types=[str])
        identity_task3 = Task("Identity function_3", FUNCTION, impl_name=lambda x: x, input_types=[str])
        merge_task_array = TaskArray('Output all inputs', tasks=[identity_task1, identity_task2, identity_task3])

        output = merge('Merge', merge_task_array, None, \
                       Task("String join", FUNCTION, impl_name=lambda x: '; '.join(x), input_types=[str]))
        print(output)
    except TigresException as e:
        print(e)

    dot_execution()
    check_final_statuses()
    end()

def check_final_statuses():
    fail_records, done_records, rerun_records, recovered_records = check('task', state=State.FAIL), \
            check('task', state=State.DONE), check('task', state=State.RERUN), check('task', state=State.RECOVERED)
    print("FAILED TASKS:\n")
    if fail_records:
        for status in check('task', state=State.FAIL):
            print("{}\n".format(status))
    print("DONE TASKS:\n")
    if done_records:
        for status in check('task', state=State.DONE):
            print("{}\n".format(status))
    print("RERUN TASKS:\n")
    if rerun_records:
        for status in check('task', state=State.RERUN):
            print("{}\n".format(status))
    print("RECOVERED TASKS:\n")
    if recovered_records:
        for status in check('task', state=State.RECOVERED):
            print("{}\n".format(status))

if __name__ == "__main__":
    # Simple Usage Here
    #run_broken_split_parallel()
    #run_fixed_split_parallel()
    run_broken_merge_parallel()
    run_broken_merge_parallel(True)
    run_fixed_merge_parallel()
    run_fixed_merge_parallel()
    #run_fixed_merge_parallel_m()
    #run_broken_merge()
    #run_fixed_merge()
    #run_broken_split()
    #run_fixed_split()