"""
7/30/14 by James Fox
"""

import os
import shutil
import subprocess
import unittest
import tigres.core.tools.log

from glob import glob
from tests.integration.recovery.resources.recover_sequence import LOG_DEST, run_broken_sequence, run_fixed_sequence, run_fixed_sequence_m
from tests.integration.recovery.resources.recover_parallel import LOG_DEST_PARALLEL, run_broken_parallel, run_fixed_parallel
from tests.integration.recovery.resources.recover_split_merge import LOG_DEST_SM, run_broken_split_parallel, run_fixed_split_parallel \
    ,run_broken_merge_parallel, run_fixed_merge_parallel, run_broken_merge, run_fixed_merge, run_broken_split, run_fixed_split \
    ,run_fixed_merge_parallel_m
from tigres.core.state.program import Program

DOT_EXEC_CMD = "python {} EXECUTION_LOCAL_THREAD"
LOG_TO_DOT_CMD = "python {} graph {} -o {}"

class Test(unittest.TestCase):
    files_to_remove = [] # intermediate files to remove (each test is responsible for adding files to this list)
    this_file_dir = ""
    resources_path = ""
    log_to_graph_path = ""

    @classmethod
    def setUpClass(self):
        """ Sets up paths """
        for logfile in glob(os.path.join(os.path.dirname(os.path.abspath(__file__)), "resources/*.log")):
            os.unlink(logfile)
        for dotfile in glob(os.path.join(os.path.dirname(os.path.abspath(__file__)), "*.dot")):
            os.unlink(dotfile)
        for dirPath in glob(os.path.join(os.path.dirname(os.path.abspath(__file__)), "resources/*data-logs")):
            shutil.rmtree(dirPath)

        file_dir_name = os.path.dirname(os.path.realpath(__file__))
        Test.this_file_dir = file_dir_name
        Test.resources_path = os.path.join(file_dir_name, 'resources')
        Test.log_to_graph_path = os.path.realpath(tigres.core.tools.log.__file__)
        Program().clear()

    @classmethod
    def tearDownClass(self):
        """
        Remove intermediate files
        """
        for file in Test.files_to_remove:
            if os.path.isfile(file):
                os.remove(file)
        for logfile in glob(os.path.join(os.path.dirname(os.path.abspath(__file__)), "resources/*.log")):
            os.unlink(logfile)
        for dotfile in glob(os.path.join(os.path.dirname(os.path.abspath(__file__)), "*.dot")):
            os.unlink(dotfile)
        for dirPath in glob(os.path.join(os.path.dirname(os.path.abspath(__file__)), "resources/*data-logs")):
            shutil.rmtree(dirPath)

        Program().clear()

    def generate_dot_outputs(self, broken_run, fixed_run, intmd_reg_dot_file, final_reg_dot_file, log_dest, \
                             intmd_log_dot_file, final_log_dot_file):
        if broken_run: broken_run()
        if fixed_run: fixed_run()
        intmd_reg_dot = os.path.join(os.path.realpath(os.curdir), intmd_reg_dot_file)
        results = subprocess.Popen(["nop", intmd_reg_dot], stdout=subprocess.PIPE)
        ref_dot_str = readFromAndWriteTo(results, os.path.join(Test.this_file_dir, final_reg_dot_file))
        subprocess.call(LOG_TO_DOT_CMD.format(Test.log_to_graph_path, log_dest, Test.this_file_dir), shell=True)
        intmd_log_dot = os.path.join(Test.this_file_dir, intmd_log_dot_file)
        results2 = subprocess.Popen(["nop", intmd_log_dot], stdout=subprocess.PIPE)
        log_dot_str = readFromAndWriteTo(results2, os.path.join(Test.this_file_dir, final_log_dot_file))

        # mark unnecessary files for removal
        # Test.files_to_remove.extend([intmd_reg_dot, intmd_log_dot, log_dest, os.path.join(Test.this_file_dir, final_reg_dot_file) \
        #                              , os.path.join(Test.this_file_dir, final_log_dot_file)])
        Test.files_to_remove.extend([intmd_reg_dot, intmd_log_dot, os.path.join(Test.this_file_dir, final_reg_dot_file) \
                                     , os.path.join(Test.this_file_dir, final_log_dot_file)])
        return ref_dot_str, log_dot_str

    def test_recover_sequence(self):
        print("testing recovery for broken sequence!\n")
        ref_dot_str, log_dot_str = Test.generate_dot_outputs(self, run_broken_sequence, run_fixed_sequence, "FixedSequence.dot", "fixed_sequence_ref.dot", \
            LOG_DEST, "broken_sequence_graph.dot", "fixed_sequence_test.dot")
        self.assertEqual(ref_dot_str, log_dot_str)

    def test_recover_parallel(self):
        print("testing recovery for broken parallel!\n")
        ref_dot_str, log_dot_str = Test.generate_dot_outputs(self, run_broken_parallel, run_fixed_parallel, "FixedParallel.dot", \
            "fixed_parallel_ref.dot", LOG_DEST_PARALLEL, "broken_parallel_graph.dot", "fixed_parallel_test.dot")
        self.assertEqual(ref_dot_str, log_dot_str)

    def test_recover_split(self):
        print("testing recovery for split task in split and merge!\n")
        ref_dot_str, log_dot_str = Test.generate_dot_outputs(self, run_broken_split, run_fixed_split, \
            "FixedSplit.dot", "fixed_split_ref.dot", \
            LOG_DEST_SM, 'broken_split_merge_graph.dot', "fixed_split_test.dot")
        self.assertEqual(ref_dot_str, log_dot_str)

    def test_recover_split_parallel(self):
        print("testing recovery for parallel split task in split and merge!\n")
        ref_dot_str, log_dot_str = Test.generate_dot_outputs(self, run_broken_split_parallel, run_fixed_split_parallel, \
            "FixedSplitParallel.dot", "fixed_split_parallel_ref.dot", \
            LOG_DEST_SM, 'broken_split_merge_graph.dot', "fixed_split_parallel_test.dot")
        self.assertEqual(ref_dot_str, log_dot_str)

    @unittest.skip("Revisit")
    def test_recover_merge(self):
        print("testing recovery for merge task in split and merge!\n")
        ref_dot_str, log_dot_str = Test.generate_dot_outputs(self, run_broken_merge, run_fixed_merge, \
            "FixedMerge.dot", "fixed_merge_ref.dot", \
            LOG_DEST_SM, 'broken_split_merge_graph.dot', "fixed_merge_test.dot")
        self.assertEqual(ref_dot_str, log_dot_str)

    def test_recover_merge_parallel(self):
        print("testing recovery for parallel merge task in split and merge!\n")
        ref_dot_str, log_dot_str = Test.generate_dot_outputs(self, run_broken_merge_parallel, run_fixed_merge_parallel, \
            "FixedMergeParallel.dot", "fixed_merge_parallel_ref.dot", \
            LOG_DEST_SM, 'broken_split_merge_graph.dot', "fixed_merge_parallel_test.dot")
        self.assertEqual(ref_dot_str, log_dot_str)

    def test_recover_merge_parallel(self):
        print("testing recovery for parallel merge task in split and merge!\n")
        ref_dot_str, log_dot_str = Test.generate_dot_outputs(self, run_broken_merge_parallel, run_fixed_merge_parallel, \
            "FixedMergeParallel.dot", "fixed_merge_parallel_ref.dot", \
            LOG_DEST_SM, 'broken_split_merge_graph.dot', "fixed_merge_parallel_test.dot")
        self.assertEqual(ref_dot_str, log_dot_str)

    def test_task_switch_recover_merge_parallel(self):
        print("testing recovery (with task name change) for parallel merge task in split and merge!\n")
        ref_dot_str, log_dot_str = Test.generate_dot_outputs(self, run_broken_merge_parallel, run_fixed_merge_parallel_m, \
            "FixedMergeParallel.dot", "fixed_switched_merge_parallel_ref.dot", \
            LOG_DEST_SM, 'broken_split_merge_graph.dot', "fixed_switched_merge_parallel_test.dot")
        self.assertEqual(ref_dot_str, log_dot_str)

    def test_task_switch_sequence(self):
        print("testing recovery (with task name change) for sequence!\n")
        ref_dot_str, log_dot_str = Test.generate_dot_outputs(self, run_broken_sequence, run_fixed_sequence_m, \
            "FixedSequence.dot", "fixed_switched_sequence_ref.dot", \
            LOG_DEST, 'broken_sequence_graph.dot', "fixed_switched_sequence_test.dot")
        self.assertEqual(ref_dot_str, log_dot_str)

    @unittest.skip("Revisit") # TODO: fix dot from log inconsistencies; related to how log statuses are parsed
    def test_2x_recover_merge_parallel(self):
        print("testing recovery following failed recovery for parallel merge task in split and merge!\n")
        run_broken_merge_parallel()
        run_fixed_merge_parallel()
        ref_dot_str, log_dot_str = Test.generate_dot_outputs(self, None, run_fixed_merge_parallel, \
            "FixedMergeParallel.dot", "fixed_2x_merge_parallel_ref.dot", \
            LOG_DEST_SM, 'broken_split_merge_graph.dot', "fixed_2x_merge_parallel_test.dot")
        self.assertEqual(ref_dot_str, log_dot_str)

    def test_2x_recover_sequence(self):
        print("testing recovery following failed recovery for sequence!\n")
        run_broken_sequence()
        run_broken_sequence(True)
        ref_dot_str, log_dot_str = Test.generate_dot_outputs(self, None, run_fixed_sequence, "FixedSequence.dot", "fixed_2x_sequence_ref.dot", \
            LOG_DEST, "broken_sequence_graph.dot", "fixed_2x_sequence_test.dot")
        self.assertEqual(ref_dot_str, log_dot_str)

def readFromAndWriteTo(readSrc, outFilePath):
        s = ""
        with open(outFilePath, 'wb') as output_file:
                while True:
                    line = readSrc.stdout.readline()
                    if not line:
                        break
                    s += bytes.decode(line, 'UTF-8')
                    output_file.write(line)
                output_file.close()
        return s

if __name__ == "__main__":
    unittest.main()




