"""
8/12/14 by James Fox
"""

from tigres import *
from tigres.utils import Execution, State, TigresException
from tests.integration.recovery.resources.LinkedList import ToyNode

import glob
import os
import shutil
import unittest

EXEC_MODE = 'EXECUTION_LOCAL_THREAD'
THIS_DIR = os.path.dirname(__file__)

ERROR_MSG = "Muhahaha"

class Test(unittest.TestCase):
    """
    Tests recovery involving complex inputs, in following cases:
        1) Recursive list ("ToyNode")
        2) Dictionary with a recursive list as a value
        3) List containing recursive list as an item
        4) Recursive list containing 2) & 3) as its items

    """
    logName = "ObjectIO"
    
    testList = [
        ToyNode(name="TestNode1", item=""),
        'Just a string',
        2048
    ]
    testDict = {
        'node': ToyNode(name="TestNode1", item=""),
        'string': 'Just a string',
        'int': 2048
    }

    @classmethod
    def setUpClass(cls):

        for logfile in glob.glob(os.path.join(os.path.dirname(os.path.abspath(__file__)), "*.log")):
            os.unlink(logfile)

    @classmethod
    def tearDownClass(cls):
        for logfile in glob.glob(os.path.join(os.path.dirname(os.path.abspath(__file__)), "*.log")):
            os.unlink(logfile)
        for logfile in glob.glob(os.path.join(os.path.dirname(os.path.abspath(__file__)), "*.dot")):
            os.unlink(logfile)
        for dirPath in glob.glob(os.path.join(os.path.dirname(os.path.abspath(__file__)), "*data-logs")):
            shutil.rmtree(dirPath)

    def runSequenceTemplate(self, input, normalFunc, recover):
        """
        Runs sequence template. Helper for tests #2, 3, and 4

        """

        this_log_dest = os.path.join(os.path.dirname(os.path.realpath(__file__)), "{}.log".format(Test.logName))
        start(name=Test.logName, log_dest=this_log_dest, execution=Execution.get(EXEC_MODE), recover=recover, recovery_log=True)
        set_log_level(Level.DEBUG)

        task_array = TaskArray('ObjectIO Tasks')
        input_array = InputArray("ObjectIO Inputs")

        variable_func = failed_method if not recover else normalFunc

        task_array.append(Task("Option0", FUNCTION, impl_name=normalFunc, input_types=[type(input)]))
        input_array.append([input])
        task_array.append(Task("Option1", FUNCTION, impl_name=variable_func, input_types=[type(input)]))
        input_array.append([PREVIOUS])
        output = sequence('Run sequence', task_array, input_array)

        #check_final_statuses()
        return output

    def test_object_input_output(self):
        """
        tests execution of sequential template with initial input and expected output of same custom Python object
        """

        logName = "ObjectIO"
        this_log_dest = os.path.join(os.path.dirname(os.path.realpath(__file__)), "{}.log".format(logName))
        start(name=logName, log_dest=this_log_dest, execution=Execution.get(EXEC_MODE), recovery=True, recovery_log=True)

        set_log_level(Level.DEBUG)

        task_array = TaskArray('ObjectIO Tasks')
        input_array = InputArray("ObjectIO Inputs")

        node = ToyNode(name="TestNode", item="")

        task_array.append(Task("Option 0", FUNCTION, impl_name=modify_node_item, input_types=[int, ToyNode]))
        input_array.append([0, node])
        task_array.append(Task("Option 1", FUNCTION, impl_name=modify_node_item, input_types=[int, ToyNode]))
        input_array.append([1, PREVIOUS])

        output = sequence('Run sequence', task_array, input_array)

        task_check = check('task', state=State.FAIL)
        assert (not task_check)
        assert isinstance(output, ToyNode)
        assert output.item == "bluebottle"
        dot_execution()

    def test_object_IO_recovery(self):
        """
        Test #1 (see Test class description)
        :return:
        """

        def runTestHelper(recover):
            """
            Helper function to switch between running broken and fixed versions of same workflow
            :param recover: recovery on or off
            """

            this_log_dest = os.path.join(os.path.dirname(os.path.realpath(__file__)), "{}.log".format(Test.logName))
            start(name=Test.logName, log_dest=this_log_dest, execution=Execution.get(EXEC_MODE), recover = recover, recovery_log=True)
            set_log_level(Level.DEBUG)

            task_array = TaskArray('ObjectIO Tasks')
            input_array = InputArray("ObjectIO Inputs")

            node = ToyNode(name="TestNode", item="")
            variable_func = failed_method if not recover else modify_node_item

            task_array.append(Task("Option0", FUNCTION, impl_name=modify_node_item, input_types=[int, ToyNode]))
            input_array.append([0, node])
            task_array.append(Task("Option1", FUNCTION, impl_name=variable_func, input_types=[int, ToyNode]))
            input_array.append([1, PREVIOUS])
            output = sequence('Run sequence', task_array, input_array)

            #check_final_statuses()
            return output

        try:
            runTestHelper(recover=False)
        except TigresException as e:
            assert str(e).split('Error:')[-1].lstrip() == ERROR_MSG
        output = runTestHelper(recover=True)
        assert isinstance(output, ToyNode)
        assert output.item == "bluebottle"
        dot_execution()


    def testComplexDict(self):
        """
        Test #2 (see Test class description)

        """
        try:
            self.runSequenceTemplate(Test.testDict, do_nothing, False)
        except TigresException as e:
            assert str(e).split('Error:')[-1].lstrip() == ERROR_MSG
        output = self.runSequenceTemplate(Test.testDict, do_nothing, True)
        assert type(output) == dict
        recovered_node = output['node']
        assert type(recovered_node) == ToyNode
        assert recovered_node.name == 'TestNode1'
        assert not recovered_node.item
        recovered_str = output['string']
        assert recovered_str == 'Just a string'
        assert output['int'] == 2048


    def testComplexList(self):
        """
        Test #3 (see Test class description)

        """

        try:
            self.runSequenceTemplate(Test.testList, do_nothing, False)
        except TigresException as e:
            assert str(e).split('Error:')[-1].lstrip() == ERROR_MSG
        output = self.runSequenceTemplate(Test.testList, do_nothing, True)
        assert type(output) == list
        recovered_node = output[0]
        assert type(recovered_node) == ToyNode
        assert recovered_node.name == 'TestNode1'
        assert not recovered_node.item
        recovered_str = output[1]
        assert recovered_str == 'Just a string'
        assert output[2] == 2048


    def testCrazyCombo(self):
        """
        Test #4 (see Test class description)

        """

        crazyRList = ToyNode(name="CrazyHead", item=Test.testDict)
        crazyRList.next = ToyNode(name="NextCrazyNode", item=Test.testList)

        try:
            self.runSequenceTemplate(crazyRList, do_nothing, False)
        except TigresException as e:
            assert str(e).split('Error:')[-1].lstrip() == ERROR_MSG
        output = self.runSequenceTemplate(crazyRList, do_nothing, True)
        assert type(output) == ToyNode
        firstItem = output.item
        assert type(firstItem) == dict
        assert type(firstItem['node']) == ToyNode
        assert firstItem['node'].name == 'TestNode1'
        assert firstItem['string'] == 'Just a string'
        assert firstItem['int'] == 2048
        
        secItem = output.next.item
        assert type(secItem) == list
        assert type(secItem[0]) == ToyNode
        assert secItem[0].name == 'TestNode1'
        assert secItem[1] == 'Just a string'
        assert secItem[2] == 2048


def modify_node_item(mod_option, node):
    """
    Modifies node's item based on provided option number (0, 1, or other)

    """
    if mod_option == 0:
        node.item += "blue"
    elif mod_option == 1:
        node.item += "bottle"
    else:
        node.item += "cap"
    return node

def do_nothing(item):
    """
    Identity function

    """
    return item

def failed_method(*args):
    raise Exception(ERROR_MSG)

def check_final_statuses():
    fail_records, done_records, rerun_records, recovered_records = check('task', state=State.FAIL), \
            check('task', state=State.DONE), check('task', state=State.RERUN), check('task', state=State.RECOVERED)
    print("FAILED TASKS:\n")
    if fail_records:
        for status in fail_records:
            print("{}\n".format(status))
    print("DONE TASKS:\n")
    if done_records:
        for status in done_records:
            print("{}\n".format(status))
    print("RERUN TASKS:\n")
    if rerun_records:
        for status in rerun_records:
            print("{}\n".format(status))
    print("RECOVERED TASKS:\n")
    if recovered_records:
        for status in recovered_records:
            print("{}\n".format(status))

"""
if __name__ == "__main__":
    Test.object_IO_recovery(Test())
"""