"""
7/2/14 by James Fox
"""

import os
import subprocess
import unittest
import glob
import tigres.core.tools.log

"""
Tests require graphviz installed
"""
DOT_EXEC_CMD = "python {} EXECUTION_LOCAL_THREAD"
LOG_TO_DOT_CMD = "python {} graph {} -o {}"

class Test(unittest.TestCase):
    files_to_remove = [] # intermediate files to remove (each test is responsible for adding files to this list)
    resources_path = ""
    outs_path = ""
    workflows_path = ""
    this_file_dir = ""
    log_to_graph_path = ""

    """ Sets up paths """
    @classmethod
    def setUpClass(self):
        file_dir_name = os.path.dirname(os.path.realpath(__file__))
        Test.this_file_dir = file_dir_name
        Test.resources_path = os.path.join(file_dir_name, 'resources')
        Test.outs_path = os.path.join(file_dir_name, '.')
        Test.workflows_path = os.path.join(file_dir_name, 'resources/workflows')
        Test.log_to_graph_path = os.path.realpath(tigres.core.tools.log.__file__)

    @classmethod
    def tearDownClass(self):
        """
        Remove intermediate files
        """
        print(Test.files_to_remove)
        for file in Test.files_to_remove:
            if os.path.exists(file):
                os.remove(file)

    def generate_dot_outputs(self, py_file, intmd_reg_dot_file, final_reg_dot_file, log_file, intmd_log_dot_file, final_log_dot_file, addt_pyfile_args=None):
        ref_dot_str = ""
        log_dot_str = ""
        py_file = os.path.join(Test.workflows_path, py_file)
        cmd = DOT_EXEC_CMD.format(py_file).split(" ")
        if addt_pyfile_args:
            cmd.extend(addt_pyfile_args)
        subprocess.call(cmd)
        intmd_reg_dot = os.path.join(os.path.realpath(os.curdir), intmd_reg_dot_file)
        results = subprocess.Popen(["nop", intmd_reg_dot], stdout=subprocess.PIPE)
        ref_dot_str += readFromAndWriteTo(results, os.path.join(Test.outs_path, final_reg_dot_file))
        log_file = os.path.join(Test.workflows_path, log_file)
        subprocess.call(LOG_TO_DOT_CMD.format(Test.log_to_graph_path, log_file, Test.this_file_dir).split(" "))
        intmd_log_dot = os.path.join(Test.this_file_dir, intmd_log_dot_file)
        results2 = subprocess.Popen(["nop", intmd_log_dot], stdout=subprocess.PIPE)
        log_dot_str += readFromAndWriteTo(results2, os.path.join(Test.outs_path, final_log_dot_file))

        # mark unnecessary files for removal
        Test.files_to_remove.extend([intmd_reg_dot, intmd_log_dot, log_file, os.path.join(Test.outs_path, final_reg_dot_file) \
                                     , os.path.join(Test.outs_path, final_log_dot_file)])
        return ref_dot_str, log_dot_str

    def test_single_sequence(self):
        print("testing single sequence!\n")
        ref_dot_str, log_dot_str = Test.generate_dot_outputs(self, 'hello_world_sequence.py', "HelloWorldSequence.dot", "hello_world_sequence.dot", \
            "hello_world_sequence.log", 'hello_world_sequence_graph.dot', "hello_world_sequence_test.dot")
        self.assertEqual(ref_dot_str, log_dot_str)

    def test_single_parallel(self):
        print("testing single parallel!\n")
        ref_dot_str, log_dot_str = Test.generate_dot_outputs(self, 'hello_world_parallel.py', "HelloWorldParallel.dot", "hello_world_parallel.dot", \
            "hello_world_parallel.log", 'hello_world_parallel_graph.dot', "hello_world_parallel_test.dot")
        self.assertEqual(ref_dot_str, log_dot_str)

    def test_split_merge(self):
        print("testing split and merge!\n")
        ref_dot_str, log_dot_str = Test.generate_dot_outputs(self, 'hello_world_split_merge.py', "HelloWorldSplitMerge.dot", "hello_world_split_merge.dot", \
            "hello_world_split_merge.log", 'hello_world_split_merge_graph.dot', "hello_world_split_merge_test.dot")
        self.assertEqual(ref_dot_str, log_dot_str)

    def test_apply_shuffle_func(self):
        print("testing apply_shuffle_func!\n")
        ref_dot_str, log_dot_str = Test.generate_dot_outputs(self, 'apply_shuffle_func.py', "ApplyShuffleFunc.dot", "apply_shuffle_func.dot", \
            "apply_shuffle_func.log", 'apply_shuffle_func_graph.dot', "apply_shuffle_func_test.dot")
        self.assertEqual(ref_dot_str, log_dot_str)

    """
    def test_apply_shuffle_exec(self):
        print("testing apply_shuffle_exec!\n")
        ref_dot_str, log_dot_str = Test.generate_dot_outputs(self, 'apply_shuffle_exec.py', "ApplyShuffleExec.dot", "apply_shuffle_exec.dot", \
            "apply_shuffle_exec.log", 'apply_shuffle_exec_graph.dot', "apply_shuffle_exec_test.dot")
        self.assertEqual(ref_dot_str, log_dot_str)
    """

    def test_loops_and_conditions(self):
        print("testing loops and conditions!\n")
        ref_dot_str, log_dot_str = Test.generate_dot_outputs(self, 'loopsAndConditions.py', "LoopsAndConditions.dot", "loops_and_conditions.dot", \
            "loopsAndConditions.log", 'loopsAndConditions_graph.dot', "loops_and_conditions_test.dot")
        self.assertEqual(ref_dot_str, log_dot_str)

def readFromAndWriteTo(readSrc, outFilePath):
        s = ""
        with open(outFilePath, 'wb') as output_file:
                while True:
                    line = readSrc.stdout.readline()
                    if not line:
                        break
                    s += bytes.decode(line, 'UTF-8')
                    output_file.write(line)
                output_file.close()
        return s

if __name__ == "__main__":
    unittest.main()




