"""
8/19/14 by James Fox
"""

import unittest
#from pprint import pprint
#from mock import patch, create_autospec, Mock
from tigres.core.utils import JSONObjectsUtil

class TestTypes(unittest.TestCase):

    def testList(self):
        print("testing list!")
        item = [1, 2]
        restored = TestTypes.deserializeAndRestore(item)
        assert item == restored
        assert dir(item) == dir(restored)

    def testBoolean(self):
        print("testing boolean!")
        true = True
        false = False
        new_true = TestTypes.deserializeAndRestore(true)
        new_false = TestTypes.deserializeAndRestore(false)
        assert true == new_true
        assert false == new_false
        assert dir(true) == dir(new_true)
        assert dir(false) == dir(new_false)

    def testNumbers(self):
        print("testing numbers!")
        num_int = 5
        num_float = 3.1416
        restored_int = TestTypes.deserializeAndRestore(num_int)
        restored_float = TestTypes.deserializeAndRestore(num_float)
        assert num_int == restored_int
        assert num_float == restored_float
        assert dir(num_int) == dir(restored_int)
        assert dir(num_float) == dir(restored_float)

    @unittest.skip("")
    def testTuple(self):
        print("testing tuple!")
        tup = (3, "batman")
        restored_tup = TestTypes.deserializeAndRestore(tup)
        assert tup == restored_tup
        assert dir(tup) == dir(restored_tup)

    def testDict(self):
        print("testing dictionary!")
        test_dict = {"name": 'Sephus'}
        restored_dict = TestTypes.deserializeAndRestore(test_dict)
        assert test_dict == restored_dict
        assert dir(test_dict) == dir(restored_dict)

    @unittest.skip("")
    def testBytes(self):
        print("testing bytes!")
        bytes = "grumpy cat".encode('UTF-8')
        restored_bytes = TestTypes.deserializeAndRestore(bytes)
        assert bytes == restored_bytes
        assert dir(bytes) == dir(restored_bytes)

    @classmethod
    def deserializeAndRestore(cls, item):
        json_output = JSONObjectsUtil.serialize(item)
        restored = JSONObjectsUtil.deserialize(json_output)
        print("original: {}, restored: {}".format(item, restored))
        return restored

    """
    Other TestTypes tests (?):
    -Exceptions
    -Functions
    """

class TestInheritance(unittest.TestCase):

    def test_basic_inheritance(self):
        """
        Tests serialization and proper deserialization of an object w/inheritance. Actual logging is skipped.
        """
        lvl_two = LevelTwo()
        json_output = JSONObjectsUtil.serialize(lvl_two)
        restored = JSONObjectsUtil.deserialize(json_output)
        TestInheritance.inheritance_helper(lvl_two, restored)

    @classmethod
    def inheritance_helper(cls, original, restored):
        """
        Performs basic inheritance tests and attribute comparisons

        :param original: original object
        :param restored: recovered object (from deserialization)
        :return:
        """
        assert dir(original) == dir(restored) # does recovered object have same attributes as original?
        assert original.__dict__ == restored.__dict__ # compares instance variables
        assert restored.fixed_level == 2
        assert restored.class_level() == 2 # tests inheritance of parent's class variable & getter method
        assert restored.message == LevelOne.message # tests inheritance of parent class variable
        assert restored.status == 'original' # tests inheritance of parent's instance variable
        restored.cheat()
        assert restored.level == 9000 # tests inheritance of parent method; update of instance variable

class LevelOne(object):
    message = "I am level 1"
    fixed_level = 1

    def __init__(self):
        self._level = LevelOne.fixed_level
        self.status = 'original'

    @classmethod
    def class_level(cls):
        return cls.fixed_level

    @classmethod
    def change_class_level(cls, value):
        cls.fixed_level = value

    @property
    def level(self):
        return self._level

    @level.setter
    def level(self, value):
        self._level = value

class LevelTwo(LevelOne):
    """
    Inherits from LevelOne
    """
    fixed_level = 2

    def __init__(self):
        LevelOne.__init__(self)
        self._level = LevelTwo.fixed_level

    def cheat(self):
        self._level = 9000


class TestRecursive(unittest.TestCase):
    """
    Tests deserialization of recursive objects
    """

    @classmethod
    def setUpClass(cls):
        TestTypes.setUpClass()

    def test_rlist(self):
        rlist = Rlist(1, Rlist(2, Rlist(3, None)))
        rlist_restored = TestTypes.deserializeAndRestore(rlist)
        print(rlist_restored)
        assert rlist == rlist_restored
        assert dir(rlist) == dir(rlist_restored)

class Rlist(object):
    """
    Recursive list
    """

    def __init__(self, first=None, rest=None):
        self.first = first
        self.rest = rest

    def __str__(self):
        """
        Returns multi-line repr of this rlist

        :rtype: str
        """
        def str_with_indent(r_list, indent=""):
            if r_list.rest == None:
                return "{}<item={}, rest=None>".format(indent, str(r_list.first))
            rstr = "{}<item={}, rest=" \
                   "\n{}" \
                   "\n{}>".format(indent, r_list.first, str_with_indent(r_list.rest, indent + '    '), indent)
            return rstr

        return "\n" + str_with_indent(self)

    def __eq__(self, other):
        if not other:
            return False
        return self.first == other.first and self.rest == other.rest

if __name__ == '__main__':
    unittest.main()