import os

__author__ = 'val'

import unittest
import tigres.core.tools.log as log
import tigres


class TestToolsLogTimes(unittest.TestCase):

    def test_times(self):
        compare_string = ["ID,START_TIME,STOP_TIME,TYPE,NAME,STATUS,EXECUTION_TIME",
        "task,Raise+letters+0,DONE,0.0008499622344970703",
        "task,Raise+letters+0#1,DONE,0.0007801055908203125",
        "task,Raise+letters+0#2,DONE,0.0006499290466308594",
        "task,Raise+letters+0#4,DONE,0.0005199909210205078",
        "task,Raise+letters+0#3,DONE,0.0004100799560546875",
        "template_parallel,Letter-raising+iteration,DONE,0.00923013687133789",
        "task,Raise+letters+1,DONE,0.0015001296997070312",
        "task,Raise+letters+1#1,DONE,0.0011899471282958984",
        "task,Raise+letters+1#2,DONE,0.000820159912109375",
        "task,Raise+letters+1#3,DONE,0.000659942626953125",
        "task,Raise+letters+1#4,DONE,0.0004999637603759766",
        "template_parallel,Letter-raising+iteration#1,DONE,0.01007986068725586",
        "task,Raise+letters+2,DONE,0.0009298324584960938",
        "task,Raise+letters+2#1,DONE,0.0009198188781738281",
        "task,Raise+letters+2#2,DONE,0.0007798671722412109",
        "task,Raise+letters+2#3,DONE,0.0005300045013427734",
        "task,Raise+letters+2#4,DONE,0.00043010711669921875",
        "template_parallel,Letter-raising+iteration#2,DONE,0.009760141372680664",
        "task,Do+nothing,DONE,0.0012900829315185547",
        "task,Do+nothing#1,DONE,0.0008299350738525391",
        "task,Do+nothing#2,DONE,0.0006999969482421875",
        "task,Do+nothing#3,DONE,0.000659942626953125",
        "task,Do+nothing#4,DONE,0.00045013427734375",
        "task,To+String,DONE,0.00012993812561035156",
        "template_merge,Fast+Decoding,DONE,0.009870052337646484",
        "program,LoopsAndConditions,DONE,0.09478998184204102", ""]

        from testfixtures import OutputCapture

        current_directory = os.path.dirname(os.path.abspath(__file__))
        log_file = "{}/test.log".format(current_directory)

        with OutputCapture() as ocapture:
            log.main(['times', '--fields=ID,START_TIME,STOP_TIME,TYPE,NAME,STATUS,EXECUTION_TIME', log_file])

        for idx, line in enumerate(ocapture.captured.split("\r\n")):
            assert(compare_string[idx] in line)

    def test_meta(self):

        from testfixtures import OutputCapture

        current_directory = os.path.dirname(os.path.abspath(__file__))
        log_file = "{}/test_meta.log".format(current_directory)

        with OutputCapture() as ocapture:
            log.main(['times', '--fields=ID,START_TIME,STOP_TIME,TYPE,NAME,STATUS,EXECUTION_TIME',"-n=all", log_file])

        for idx, line in enumerate(ocapture.captured.split("\r\n")):
            if idx == 0:
                # All possible metadata should be in the header (lowercase)
                self.assertEqual("ID,START_TIME,STOP_TIME,TYPE,NAME,STATUS,EXECUTION_TIME,execution,version", line)
            else:
                # The cores and verison metadata should only exist in one workflow
                if line.find("d75c47b0-67eb-49f6-a281-7cf1014d4122") > -1:
                    self.assertTrue(line.find("ExecutionPluginDistributeProcess,0.2.0-rc4")) == -1
                else:
                    self.assertTrue(line.find("ExecutionPluginDistributeProcess,0.2.0-rc4"))


    def test_custom_meta(self):
            compare_string = """"""
            from testfixtures import OutputCapture

            current_directory = os.path.dirname(os.path.abspath(__file__))
            log_file = "{}/test_meta.log".format(current_directory)

            with OutputCapture() as ocapture:
                log.main(['times', '--fields=ID,START_TIME,STOP_TIME,TYPE,NAME,STATUS,EXECUTION_TIME', "-n=all",
                          "-m=version,cores", log_file])

            for idx, line in enumerate(ocapture.captured.split("\r\n")):
                if idx == 0:
                    # All possible metadata should be in the header (lowercase)
                    self.assertEqual("ID,START_TIME,STOP_TIME,TYPE,NAME,STATUS,EXECUTION_TIME,cores,version",line)
                else:
                    # The cores and verison metadata should only exist in one workflow
                    if line.find("d75c47b0-67eb-49f6-a281-7cf1014d4122")>-1:
                        self.assertTrue(line.find("50,0.2.0-rc4"))==-1
                    else:
                        self.assertTrue(line.find("50,0.2.0-rc4"))



if __name__ == '__main__':
    unittest.main()
